package com.wizarpos.mdbtest;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.wizarpos.util.MDBUtils;

public class HomeFragment extends Fragment implements View.OnClickListener{

	private Button openBtn, closeBtn, btnGetVersion, btnUpdateFw, startBtn, btnFactoryMode,
				   btnTest, btnGetHardwareVersion, btnDiagnoseHardware;
	private Spinner spnWhiteDemon, spnMdbLevel;
	private String spnItemWhiteDemon, spnItemMdbLevel;
	private Context context;

	private static final String TAG = "HomeFragment";
	private DataSendListener dataSendListener;

//    public boolean isSlave = false;

	@Nullable
	@Override
	public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
		View view = inflater.inflate(R.layout.fragment_home, container, false);

		spnWhiteDemon = (Spinner) view.findViewById(R.id.spn_white_demon);
		spnMdbLevel = view.findViewById(R.id.spn_mdb_level);

		openBtn = (Button) view.findViewById(R.id.open);
		closeBtn = (Button) view.findViewById(R.id.close);
		startBtn = (Button) view.findViewById(R.id.btn_start);
		btnGetVersion = (Button) view.findViewById(R.id.btn_get_version);
		btnUpdateFw = (Button) view.findViewById(R.id.btn_update_fw);
		btnFactoryMode = (Button) view.findViewById(R.id.btn_factory_mode);
		btnTest = view.findViewById(R.id.btn_test);
		btnGetHardwareVersion = view.findViewById(R.id.btn_get_hardware_version);
		btnDiagnoseHardware = view.findViewById(R.id.btn_diagnose_hardware);

		openBtn.setOnClickListener(this);
		closeBtn.setOnClickListener(this);
		startBtn.setOnClickListener(this);
		btnGetVersion.setOnClickListener(this);
		btnUpdateFw.setOnClickListener(this);
		btnFactoryMode.setOnClickListener(this);
		btnTest.setOnClickListener(this);
		btnGetHardwareVersion.setOnClickListener(this);
		btnDiagnoseHardware.setOnClickListener(this);

		context = getContext();
		makeSpinner();

		return view;
	}

	@Override
	public void onAttach(Context context) {
		super.onAttach(context);
		try{
			dataSendListener = (DataSendListener) context;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public void makeSpinner(){
		ArrayAdapter<CharSequence> adapterWhiteDemon = ArrayAdapter.createFromResource(context,
				R.array.white_demon, android.R.layout.simple_spinner_item);
		adapterWhiteDemon.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spnWhiteDemon.setAdapter(adapterWhiteDemon);
		spnWhiteDemon.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
				spnItemWhiteDemon = spnWhiteDemon.getSelectedItem().toString();
				dataSendListener.onStringValueSent(MDBUtils.TYPE_SPN_FIRMWARE_ITEM, spnItemWhiteDemon);
			}
			@Override
			public void onNothingSelected(AdapterView<?> adapterView) {

			}
		});

		ArrayAdapter<CharSequence> adapterMdbLevel = ArrayAdapter.createFromResource(context,
				R.array.spn_mdb_level, android.R.layout.simple_spinner_item);
		adapterMdbLevel.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spnMdbLevel.setAdapter(adapterMdbLevel);
		spnMdbLevel.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
				spnItemMdbLevel = spnMdbLevel.getSelectedItem().toString();
				int nMdbLevel = Integer.parseInt(spnItemMdbLevel);
				dataSendListener.onIntValueSent(MDBUtils.TYPE_SPN_MDB_LEVEL, nMdbLevel);
			}
			@Override
			public void onNothingSelected(AdapterView<?> adapterView) {

			}
		});

	}

	@Override
	public void onClick(View view) {
		switch (view.getId()) {
			case R.id.open:
				dataSendListener.onOpenClicked();
				break;
			case R.id.close:
				dataSendListener.onCloseClicked();
				break;
			case R.id.btn_get_version:
				dataSendListener.onGetVersionClicked();
				break;
			case R.id.btn_update_fw:
				dataSendListener.onUpdateFwClicked();
				break;
			case R.id.btn_factory_mode:
				dataSendListener.onFactoryModeClicked();
				break;
			case R.id.btn_test:
				dataSendListener.onTestClicked();
				break;
			case R.id.btn_get_hardware_version:
				dataSendListener.onGetHardwareVersionClicked();
				break;
			case R.id.btn_diagnose_hardware:
				dataSendListener.onDiagnoseHardwareClicked();
				break;
		}
	}
}
