package com.wizarpos.mdbtest;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.os.SystemClock;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.ActionBarDrawerToggle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.view.GravityCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;

import com.cloudpos.DeviceException;
import com.cloudpos.OperationListener;
import com.cloudpos.OperationResult;
import com.cloudpos.POSTerminal;
import com.cloudpos.TimeConstants;
import com.cloudpos.card.Card;
import com.cloudpos.jniinterface.SerialPortInterface;
import com.cloudpos.jniinterface.Stm32ispInterface;
import com.cloudpos.rfcardreader.RFCardReaderDevice;
import com.cloudpos.rfcardreader.RFCardReaderOperationResult;
import com.cloudpos.smartcardreader.SmartCardReaderDevice;
import com.google.android.material.navigation.NavigationView;
import com.wizarpos.util.ByteConvertStringUtil;
import com.wizarpos.util.LogHelper;
import com.wizarpos.util.MDBUtils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.Locale;
import java.util.Queue;

public class MainActivity extends AppCompatActivity implements DataSendListener {

    private DrawerLayout drawerLayout;
    private ActionBarDrawerToggle toggle;
    private TextView textView;

    private Handler handler;
    private int pulseFrequency = 1; //default: 1
    private int pulseLatency = 300; //default: 300
    private int pulseDuration = 200; //default: 200
    private int pulseVoltage = 0; //default: 0
    private int balance = 100; //default: 100
    private int currentVersion = 0;
    private String firmwareName = "";
    private int mdbLevel = 2;
    private boolean factoryMode = false;
    private int currentSend;
    private Fragment selectedFragment = null;

    public boolean readCardDone = true;
    public Queue<Message> msgQueueReadCard = new LinkedList<>();
    private Context context;

    private Card rfCard;
    private RFCardReaderDevice rfCardReaderDevice = null;
    private SmartCardReaderDevice smartCardReaderDevice = null;

    private final String TAG = "MDBTest TAG";
    private final boolean TRIGGER_PULSE_ONE_SHOT = true;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        textView = (TextView)findViewById(R.id.textviewaa);
        textView.setMovementMethod(ScrollingMovementMethod.getInstance());
        context = this;

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        drawerLayout = findViewById(R.id.drawer_layout);
        NavigationView navigationView = findViewById(R.id.nav_view);
        toggle = new ActionBarDrawerToggle(this, drawerLayout, toolbar, R.string.nav_drawer_open, R.string.nav_drawer_close);
        drawerLayout.addDrawerListener(toggle);
        toggle.syncState();
        selectedFragment = new HomeFragment();
//        getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, selectedFragment).commit();
        navigationView.setNavigationItemSelectedListener(new NavigationView.OnNavigationItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(@NonNull MenuItem item) {
                if(item.getItemId() == R.id.nav_home){
                    selectedFragment = new HomeFragment();
                } else if (item.getItemId() == R.id.nav_pulse) {
                    selectedFragment = new PulseFragment();
                } else if (item.getItemId() == R.id.nav_transaction) {
                    selectedFragment = new TransactionFragment();
                }
                if(selectedFragment != null){
                    FragmentManager fragmentManager = getSupportFragmentManager();
                    fragmentManager.beginTransaction().replace(R.id.fragment_container, selectedFragment).commit();
                }
                drawerLayout.closeDrawer(GravityCompat.START);
                return true;
            }
        });
        if(savedInstanceState == null){
            getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, selectedFragment).commit();
            navigationView.setCheckedItem(R.id.nav_home);
        }
        handler = new Handler(new Handler.Callback() {
            @Override
            public boolean handleMessage(Message msg) {
                switch (msg.what) {
                    case MDBUtils.ENABLE_ALL_UI:
                        enableAllUi();
                        break;
                    case MDBUtils.DISABLE_ALL_UI:
                        disableAllUi();
                        break;
                    case MDBUtils.BLACK_LOG:
                        LogHelper.appendBlackMsg((String) msg.obj, textView);
                        break;
                    case MDBUtils.RED_LOG:
                        LogHelper.appendREDMsg((String) msg.obj, textView);
                        break;
                    case MDBUtils.GREEN_LOG:
                        LogHelper.appendGreenMsg((String) msg.obj, textView);
                        break;
                    case MDBUtils.DIALOG_CONFIRM:
                        Item itemDialog = (Item) msg.obj;
                        AlertDialog dialog = new AlertDialog.Builder(context)
                                .setTitle("Please confirm item and amount")
                                .setMessage("Vend product\n " + "Amount: $" + itemDialog.getItemPrice() + ",Item: " + itemDialog.getItemId())
                                .setNegativeButton("Deny", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        sendVendDedied();
                                        readCardDone = true;
                                        dialog.dismiss();
                                        processReadcard();
                                    }
                                })
                                .setPositiveButton("Approve", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        sendVendApproved(itemDialog);
                                        readCardDone = true;
                                        dialog.dismiss();
                                    }
                                }).create();
                        dialog.show();
                        break;
                    case MDBUtils.DIALOG_ITEM:
                        AlertDialog dialog1 = new AlertDialog.Builder(context)
                                .setTitle("Dispense")
                                .setMessage(msg.obj + "")
                                .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        processReadcard();
                                        dialog.dismiss();
                                    }
                                }).create();
                        dialog1.show();
                        break;
                    case MDBUtils.DIALOG_READCARD:
                        Message message = new Message();
                        message.what = msg.what;
                        message.obj = msg.obj;
                        msgQueueReadCard.add(message);
                        if(readCardDone) {
                            processReadcard();
                        }
                        break;
                    case MDBUtils.TEST_MSG:
                        if (rfCardReaderDevice == null) {
                            rfCardReaderDevice = (RFCardReaderDevice) POSTerminal.getInstance(context)
                                    .getDevice("cloudpos.device.rfcardreader");
                        }
                        try {
                            rfCardReaderDevice.open();
                            SystemClock.sleep(5000);
                            readCardDone = true;
                            rfCardReaderDevice.close();
                        } catch (DeviceException e) {
                            Log.e(TAG, "open rfCardReaderDevice failed !");
                            e.printStackTrace();
                        }
                        break;
                }
                return false;
            }
        });
        open();
    }

    public void processReadcard(){
        if(msgQueueReadCard.isEmpty()){
            readCardDone = true;
            return;
        }
        Log.d(TAG, "processing read card");
        readCardDone = false;
        Message msgReadCard = msgQueueReadCard.poll();
        AlertDialog cardDialog = new AlertDialog.Builder(context)
                .setTitle("Read card")
                .setMessage("Please swipe card.")
                .create();
        cardDialog.show();
        Item item = (Item) msgReadCard.obj;
        Log.d(TAG, "Read item data " + item.getItemPrice() + "," + item.getItemId());
        if (rfCardReaderDevice == null) {
            rfCardReaderDevice = (RFCardReaderDevice) POSTerminal.getInstance(context)
                    .getDevice("cloudpos.device.rfcardreader");
        }
        if (smartCardReaderDevice == null) {
            smartCardReaderDevice = (SmartCardReaderDevice) POSTerminal.getInstance(context)
                    .getDevice("cloudpos.device.smartcardreader");
        }
        try {
            rfCardReaderDevice.open();
            try {
                OperationListener listener = new OperationListener() {
                    @Override
                    public void handleResult(OperationResult arg0) {
                        try {
                            if (arg0.getResultCode() == OperationResult.SUCCESS) {
                                rfCard = ((RFCardReaderOperationResult) arg0).getCard();
                                handler.obtainMessage(MDBUtils.BLACK_LOG, "amount: $" + item.getItemPrice() + ",item: " + item.getItemId()).sendToTarget();
                                handler.obtainMessage(MDBUtils.DIALOG_CONFIRM, item).sendToTarget();
                            } else {
                                Log.e(TAG, "find_card_failed");
                                readCardDone = true;
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        } finally {
                            try {
                                rfCardReaderDevice.close();
                                cardDialog.dismiss();
                            } catch (DeviceException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                };
                rfCardReaderDevice.listenForCardPresent(listener, TimeConstants.FOREVER);
            } catch (DeviceException e) {
                e.printStackTrace();
            }
        } catch (DeviceException e) {
            Log.e(TAG, "open rfCardReaderDevice failed !");
            e.printStackTrace();
        }
    }

    public void testMdbSlaveRead() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                Item item = new Item();
                handler.obtainMessage(MDBUtils.BLACK_LOG, "Init, waiting master command...").sendToTarget();
//                while (isSlave) {
                while (true) {
                    byte[] byteArr = new byte[1];
                    // read start flag 0x09, 1 bit
                    int readResult = SerialPortInterface.read(byteArr, byteArr.length, -1);
                    Log.d(TAG, "read 1 byte success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                    if(ByteConvertStringUtil.buf2StringCompact(byteArr).equals("0D")){
                        continue;
                    }
                    if (readResult >= 0) {
                        if (ByteConvertStringUtil.buf2StringCompact(byteArr).equals("09")) {
                            // read data length, 1 bit
                            readResult = SerialPortInterface.read(byteArr, byteArr.length, 200);
                            if (readResult > 0) {
                                Log.d(TAG, "read length success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                                String lenthHex = ByteConvertStringUtil.buf2StringCompact(byteArr);
                                BigInteger length = new BigInteger(lenthHex, 16);
                                // read data
                                byteArr = new byte[length.intValue()];
                                readResult = SerialPortInterface.read(byteArr, byteArr.length, 200);
                                if (readResult > 0) {
                                    Log.i(TAG, "read success: " + ByteConvertStringUtil.buf2StringCompact(byteArr));
                                } else {
                                    Log.e(TAG, "read failed");
                                }
                                processData(item, byteArr);
                            }
                        }
                    }
//                    else if (readResult == -262254) {
//                        handler.obtainMessage(MDBUtils.RED_LOG, "serialport read timeout").sendToTarget();
//                    } else {
//                        handler.obtainMessage(MDBUtils.RED_LOG, "serialport read failed = " + readResult).sendToTarget();
//                    }
                    if (   !compareByteArrayHead(byteArr, new byte[]{0x01, 0x20, 0x00}, 3)
                        && !compareByteArrayHead(byteArr, new byte[]{0x0D}, 1)) {
                        SystemClock.sleep(500);
                    }
                }
//                close();
            }
        }).start();
    }

    private void processData(Item item, byte[] readBytes){
//        Log.i(TAG, "readBytes:  " + ByteConvertStringUtil.buf2StringCompact(readBytes));
        if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x10, 0x10}, 3)) {
            sendReset();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x11, 0x00}, 3)) { //Setup config data
            sendSetupConfig(readBytes, item);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x11, 0x01}, 3)) { // Setup Max/Min Price
            sendSetupPrice();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x17, 0x00}, 3)) { //Expansion ID
            sendExpansionID();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x00}, 3)
                || compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x01}, 3)) {//Enable Reader
            sendEnableReader();
            startTransaction();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, 0x00}, 2)) {
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Ready! Please input item ID").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x15, 0x00}, 3)){ //Revalue approved
            sendRevalueApproved();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x15, 0x01}, 3)){ //Revalue limited
            sendRevalueLimit();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x00}, 3)) { // vend request
            //e.g. 00 13 00 00 A0 00 01 B4 98
            //amount: 00 A0
            //id: 00 01
            item.setItemAmount(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[3], readBytes[4]}).replace(" ", ""));
//            item.setItemId(sp.substring(15, 21).replace(" ", ""));
            item.setItemId(ByteConvertStringUtil.buf2StringCompact(new byte[]{readBytes[5], readBytes[6]}).replace(" ", ""));
            item.calculatePrice();
            BigInteger itemid = new BigInteger(item.getItemId(), 16);
            item.setItemId(itemid.toString());
            handler.obtainMessage(MDBUtils.DIALOG_READCARD, item).sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x02}, 3)) { //vend dispatch
            sendVendSuccess(readBytes);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x13, 0x04}, 3)) { //vend session end
            sendSessionEnd();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, 0x14, 0x02}, 3)){ //reader cancel
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Reader cancelled").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x03}, 3)) { //set duration
            handler.obtainMessage(MDBUtils.BLACK_LOG, "set pulse duration succeed").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x04}, 3)) { //set voltage
            handler.obtainMessage(MDBUtils.BLACK_LOG, "set pulse voltage succeed").sendToTarget();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x90, 0x05}, 3)){ //factory mode
            if(factoryMode) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "enter factory mode succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "quit factory mode succeed").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x92, 0x00}, 3)) { //get version
            handler.obtainMessage(MDBUtils.BLACK_LOG, "get result:" +
                    ByteConvertStringUtil.buf2StringCompact(readBytes)).sendToTarget();
            printVersionName(readBytes);
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x94}, 2)) { //diagnose hardware
            if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x94, 0x00}, 3)) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "hardware diagnose test received").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "hardware diagnose test open failed").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x93}, 2)) { //get hardware version
            if(compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x93, 0x00}, 3)) {
              printHardwareVersion(readBytes);
            } else {
                handler.obtainMessage(MDBUtils.RED_LOG, "get hardware version failed!").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x00, (byte)0x13, 0x01}, 3)) {  // vend denied
            Log.d(TAG, "00 13 01, vending auto cancel, denied");
            sendVendDedied();
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, 0x20, 0x00}, 3)) { //trigger pulse
            //receive: 09 04 01 20 00 DF 0D
            --currentSend;
            if (currentSend == 0) {
                handler.obtainMessage(MDBUtils.ENABLE_ALL_UI).sendToTarget();
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse " + pulseFrequency + " times succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse " + (pulseFrequency - currentSend)).sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x21}, 2)) { //trigger pulse one shot resp
            if(compareByteArrayHead(readBytes, new byte[]{0x01, (byte)0x21, 0x00}, 3)) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse one shot succeed").sendToTarget();
            } else {
                handler.obtainMessage(MDBUtils.RED_LOG, "trigger pulse one shot failed!").sendToTarget();
            }
        } else if (compareByteArrayHead(readBytes, new byte[]{0x0d}, 1)){
            Log.d(TAG, "0D, end of data");
        } else {
            handler.obtainMessage(MDBUtils.BLACK_LOG, "unknown data: " +
                    ByteConvertStringUtil.buf2StringCompact(readBytes)).sendToTarget();
        }
    }

    private boolean compareByteArrayHead(byte[] byteArray, byte[] bytesHead, int nBytes){
        byte[] tmpBytesHead = Arrays.copyOfRange(byteArray, 0, nBytes);
        return Arrays.equals(tmpBytesHead, bytesHead);
    }

    private void sendReset(){
        byte[] dataBytes = new byte[2];
        dataBytes[0] = 0;
        dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Reset 0.0%. wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Reset 0.0%").sendToTarget();
        }
    }
    //readBytes = "00 11 00 02 00 00 01 14"
    private void sendSetupConfig(byte[] readBytes, Item item){
        byte[] dataBytes = new byte[9];
        dataBytes[0] = 0x01; //Reader Config Data
        dataBytes[1] = readBytes[3]; //Reader Feature Level
        dataBytes[2] = 0x11; //Country Code High
        dataBytes[3] = (byte)0x56; // Country Code Low
        dataBytes[4] = 0x01; //Scale Factor (x when calculating price)
        item.setPriceX(Integer.toHexString(dataBytes[4]));
        dataBytes[5] = 0x02; //Decimal Places (y when calculating price)
        item.setPriceY(Integer.toHexString(dataBytes[5]));
        dataBytes[6] = 0x59; //Application Maximum Response Time
        dataBytes[7] = 0x0d; // Miscellaneous Options
        dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Setup config data 20.0%. wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Setup config data 20.0%").sendToTarget();
        }
    }
    private void sendSetupPrice(){
        byte[] dataBytes = new byte[1];
        dataBytes[0] = 0;
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Setup Max/Min Price 50.0%. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Setup Max/Min Price 50.0%").sendToTarget();
        }
    }
    //sp = "00 17 00 49 44 53 30 30 30 30 31 30 30 30 36 36 30 32 49 44 53 4d 44 42 56 4d 43 43 4f 4e 30 32 21"
    private void sendExpansionID(){
        byte[] dataBytes = new byte[31];
        dataBytes[0] = 0x09; //Peripheral ID
        dataBytes[1] = 0x77;
        dataBytes[2] = 0x7a;
        dataBytes[3] = 0x70; //dataBytes[1-3]: Manufacturer Code
        dataBytes[4] = 0x00;
        dataBytes[5] = 0x00;
        dataBytes[6] = 0x00;
        dataBytes[7] = 0x00;
        dataBytes[8] = 0x00;
        dataBytes[9] = 0x00;
        dataBytes[10] = 0x00;
        dataBytes[11] = 0x00;
        dataBytes[12] = 0x00;
        dataBytes[13] = 0x00;
        dataBytes[14] = 0x00;
        dataBytes[15] = 0x01;//dataBytes[4-15]: Serial Number
        dataBytes[16] = 0x00;
        dataBytes[17] = 0x00;
        dataBytes[18] = 0x00;
        dataBytes[19] = 0x00;
        dataBytes[20] = 0x00;
        dataBytes[21] = 0x00;
        dataBytes[22] = 0x00;
        dataBytes[23] = 0x00;
        dataBytes[24] = 0x00;
        dataBytes[25] = 0x00;
        dataBytes[26] = 0x00;
        dataBytes[27] = 0x01;//dataBytes[16-27]: Model Number
        dataBytes[28] = 0x00;
        dataBytes[29] = 0x01;//dataBytes[28-29]: Software Version
        dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Expansion ID 60.0%. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Expansion ID 60.0%").sendToTarget();
        }
    }
    private void sendEnableReader(){
        byte[] dataBytes = new byte[1];
        dataBytes[0] = 0;
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Enable Reader 80.0%. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Enable Reader 80.0%").sendToTarget();
        }
    }
    private void sendRevalueApproved(){
        byte[] dataBytes = new byte[1];
        dataBytes[0] = 0x0d;
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Revalue approved 85%. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Revalue approved 85%").sendToTarget();
        }
    }
    private void sendRevalueLimit(){
        byte[] dataBytes = new byte[4];
        dataBytes[0] = 0x0f; //Revalue Limit Amount
        dataBytes[1] = 0x27;
        dataBytes[2] = 0x10;//1-2: Revalue Limit Amount
        dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "Revalue limit request 90.0%. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "Revalue limit request 90.0%").sendToTarget();
        }
    }

    private void sendVendSuccess(byte[] readBytes){
        byte[] dataBytes = new byte[1];
        dataBytes[0] = 0;
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        int itemIdHigh = readBytes[3];
        int itemIdLow = readBytes[4];
        int itemId = ((itemIdHigh & 0xFF) << 8) | (itemIdLow & 0xFF);
        if (wresult >= 0) {
            Log.d(TAG, "Vend success! wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.DIALOG_ITEM, "Vend success! item:" + itemId).sendToTarget();
        }
    }
    private void sendSessionEnd(){
        byte[] dataBytes = new byte[2];
        dataBytes[0] = 0x07; //End Session
        dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "finish. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "finish.").sendToTarget();
            SystemClock.sleep(2000);
            startTransaction();
        }
    }
    private void sendVendApproved(Item item){
        byte[] dataBytes = new byte[4];
        dataBytes[0] = 0x05;
        dataBytes[1] = (byte) Integer.parseInt(item.getItemAmount().substring(0,2), 16);
        dataBytes[2] = (byte) Integer.parseInt(item.getItemAmount().substring(2), 16);
        dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "approved! wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "approved!").sendToTarget();
        }
    }
    private void sendVendDedied(){
        byte[] dataBytes = new byte[2];
        dataBytes[0] = 0x06;
        dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "denied! wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "denied!").sendToTarget();
        }
    }

    //begin session
    private void startTransaction() {
        byte[] dataBytes = new byte[11];
        dataBytes[0] = 0x03; //Begin Session
        if(balance < 0) {
            dataBytes[1] = 0x03;
            dataBytes[2] = (byte) 0xE8; //1-2: Funds/balance Available
        } else {
            dataBytes[1] = (byte)(balance >> 8 & 0xFF);
            dataBytes[2] = (byte)(balance & 0xFF);
        }
//        Log.i(TAG, " dataBytes[1] " + String.valueOf(dataBytes[1]) + " dataBytes[2] " + String.valueOf(dataBytes[2]));
//        Log.i(TAG, "balance : " + ((dataBytes[1] << 8 & 0xFF) | (dataBytes[2] & 0xFF)));
        dataBytes[3] = (byte)0xFF;
        dataBytes[4] = (byte)0xFF;
        dataBytes[5] = (byte)0xFF;
        dataBytes[6] = (byte)0xFF; //3-6: Payment media ID
        dataBytes[7] = (byte)0x00; //Payment Type
        dataBytes[8] = (byte)0x00;
        dataBytes[9] = (byte)0x00; //8-9: Payment Data
        dataBytes = MDBUtils.addMdbCheckSum(dataBytes);
        byte[] wbytes = MDBUtils.mergePacket(0, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "begin session. wbytes: " + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "begin session").sendToTarget();
        }
    }
    private void sendReaderCancelled(String sp){
        byte[] dataBytes = new byte[1];
        dataBytes[0] = 0x08;
        byte[] wbytes = MDBUtils.mergePacket(1, dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "cancelled. wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "cancelled").sendToTarget();
        }
    }
    private void sendTriggerPulse(){ //09 03 00 20 E0 0D resp: 09 04 01 20 00 DF 0D
        byte[] dataBytes = new byte[1];
        dataBytes[0] = 0x20;
        byte[] wbytes = MDBUtils.mergePacket(0,dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send trigger pulse succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
    }
    private void sendTriggerPulseOneShot(){
        byte[] voltageByte = new byte[4];
        voltageByte[0] = (byte)(pulseVoltage & 0xFF);
        voltageByte[1] = (byte)((pulseVoltage >> 8) & 0xFF);
        voltageByte[2] = (byte)((pulseVoltage >> 16) & 0xFF);
        voltageByte[3] = (byte)((pulseVoltage >> 24) & 0xFF);
        byte[] durationByte = new byte[4];
        durationByte[0] = (byte)(pulseDuration & 0xFF);
        durationByte[1] = (byte)((pulseDuration >> 8) & 0xFF);
        durationByte[2] = (byte)((pulseDuration >> 16) & 0xFF);
        durationByte[3] = (byte)((pulseDuration >> 24) & 0xFF);
        byte[] intervalByte = new byte[4];
        intervalByte[0] = (byte)(pulseLatency & 0xFF);
        intervalByte[1] = (byte)((pulseLatency >> 8) & 0xFF);
        intervalByte[2] = (byte)((pulseLatency >> 16) & 0xFF);
        intervalByte[3] = (byte)((pulseLatency >> 24) & 0xFF);
        byte[] frequencyByte = new byte[4];
        frequencyByte[0] = (byte)(pulseFrequency & 0xFF);
        frequencyByte[1] = (byte)((pulseFrequency >> 8) & 0xFF);
        frequencyByte[2] = (byte)((pulseFrequency >> 16) & 0xFF);
        frequencyByte[3] = (byte)((pulseFrequency >> 24) & 0xFF);

        byte[] dataBytes = new byte[17];
        dataBytes[0] = 0x21;
        dataBytes[1] = voltageByte[0];
        dataBytes[2] = voltageByte[1];
        dataBytes[3] = voltageByte[2];
        dataBytes[4] = voltageByte[3];
        dataBytes[5] = durationByte[0];
        dataBytes[6] = durationByte[1];
        dataBytes[7] = durationByte[2];
        dataBytes[8] = durationByte[3];
        dataBytes[9] = intervalByte[0];
        dataBytes[10] = intervalByte[1];
        dataBytes[11] = intervalByte[2];
        dataBytes[12] = intervalByte[3];
        dataBytes[13] = frequencyByte[0];
        dataBytes[14] = frequencyByte[1];
        dataBytes[15] = frequencyByte[2];
        dataBytes[16] = frequencyByte[3];

        byte[] wbytes = MDBUtils.mergePacket(0,dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send trigger pulse one shot succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
    }
    private void sendSetPulseDuration(int duration){ //09 08 00 90 03 C8 00 00 00 A5 0D resp: 09 05 01 90 03 00 6C 0D
        byte[] durationByte = new byte[4];
        durationByte[0] = (byte)(duration & 0xFF);
        durationByte[1] = (byte)((duration >> 8) & 0xFF);
        durationByte[2] = (byte)((duration >> 16) & 0xFF);
        durationByte[3] = (byte)((duration >> 24) & 0xFF);

        byte[] dataBytes = new byte[6];
        dataBytes[0] = (byte)0x90;
        dataBytes[1] = (byte)0x03;
        dataBytes[2] = durationByte[0];
        dataBytes[3] = durationByte[1];
        dataBytes[4] = durationByte[2];
        dataBytes[5] = durationByte[3];//2-5: duration
        byte[] wbytes = MDBUtils.mergePacket(0,dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send set pulse duration succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "setting pulse duration").sendToTarget();
        }
        SystemClock.sleep(200);
    }
    private void sendSetPulseVoltage(int voltage){ //09 08 00 90 04 00 00 00 00 6C 0D resp: 09 05 01 90 04 00 6B 0D
        byte[] voltageByte = new byte[4];
        voltageByte[0] = (byte)(voltage & 0xFF);
        voltageByte[1] = (byte)((voltage >> 8) & 0xFF);
        voltageByte[2] = (byte)((voltage >> 16) & 0xFF);
        voltageByte[3] = (byte)((voltage >> 24) & 0xFF);

        byte[] dataBytes = new byte[6];
        dataBytes[0] = (byte)0x90;
        dataBytes[1] = (byte)0x04;
        dataBytes[2] = voltageByte[0];
        dataBytes[3] = voltageByte[1];
        dataBytes[4] = voltageByte[2];
        dataBytes[5] = voltageByte[3]; //2-5: voltage
        byte[] wbytes = MDBUtils.mergePacket(0,dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send set pulse voltage succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "setting pulse voltage").sendToTarget();
        }
        SystemClock.sleep(200);
    }
    private void sendEnterFactoryMode(){
        byte[] dataBytes = new byte[6];
        dataBytes[0] = (byte)0x90;
        dataBytes[1] = (byte)0x05;
        dataBytes[2] = (byte)0x01;
        dataBytes[3] = (byte)0x00; //2-3: factory mode code
        dataBytes[4] = (byte)0x00;
        dataBytes[5] = (byte)0x00;
        byte[] wbytes = MDBUtils.mergePacket(0,dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send enter factory succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
//            handler.obtainMessage(MDBUtils.BLACK_LOG, "send enter factory succeed !").sendToTarget();
            factoryMode = true;
            View view = selectedFragment.getView();
            if(view != null){
                Button button = view.findViewById(R.id.btn_factory_mode);
                button.setText("quit factory mode");
            }
        }
        SystemClock.sleep(200);
    }
    private void sendQuitFactoryMode(){
        byte[] dataBytes = new byte[6];
        dataBytes[0] = (byte)0x90;
        dataBytes[1] = (byte)0x05;
        dataBytes[2] = (byte)0x00;
        dataBytes[3] = (byte)0x00; //2-3: factory mode code
        dataBytes[4] = (byte)0x00;
        dataBytes[5] = (byte)0x00;
        byte[] wbytes = MDBUtils.mergePacket(0,dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send quit factory succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
//            handler.obtainMessage(MDBUtils.BLACK_LOG, "send quit factory succeed !").sendToTarget();
            factoryMode = false;
            View view = selectedFragment.getView();
            if(view != null){
                Button button = view.findViewById(R.id.btn_factory_mode);
                button.setText("enter factory mode");
            }
        }
        SystemClock.sleep(200);
    }

    private void sendHardwareDiagnoseTest(){
        byte[] dataBytes = new byte[1];
        dataBytes[0] = (byte)0x94;
        byte[] wbytes = MDBUtils.mergePacket(0,dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send hardware diagnose test succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
            handler.obtainMessage(MDBUtils.BLACK_LOG, "send hardware diagnose test succeed !").sendToTarget();
        }
        SystemClock.sleep(200);
    }

    //09 03 00 92 6E 0D resp: 09 06 01 92 00 00 0E 5F 0D
    //if version number is 20, resp: 09 06 01 92 00 00 14 59 0D
    private void sendVersionRequest() {
        byte[] wbytes = {0x09, 0x03, 0x00, (byte) 0x92, (byte) 0x6E, (byte) 0x0D};
        int write = SerialPortInterface.write(wbytes, 0, wbytes.length);
        handler.obtainMessage(MDBUtils.BLACK_LOG, "Getting version, wait " + write).sendToTarget();
        if (write >= 0) {
            Log.d(TAG, "send get version request succeed !" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
        SystemClock.sleep(200);
    }
    private void sendHardwareVersionRequest(){//09 03 00 93 6D 0D resp: 09 05 01 93 00 04 68 0D
        byte[] dataBytes = new byte[1];
        dataBytes[0] = (byte) 0x93;
        byte[] wbytes = MDBUtils.mergePacket(0,dataBytes);
        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "send get hardware version succeed ! wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
    }

    private void printHardwareVersion(byte[] readBytes){
//        String version = sp.substring(9, 11);
//        int hardwareVersionNo = Integer.parseInt(version, 16);
        int hardwareVersionNo = readBytes[3];
        handler.obtainMessage(MDBUtils.BLACK_LOG, "hardware version: " + hardwareVersionNo).sendToTarget();
    }

    private void testMdbMasterSend(){
        new Thread(new Runnable(){
            @Override
            public void run() {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "Init, sending reset request").sendToTarget();
                Log.d(TAG, "Init, sending reset request");
                boolean flagEnd = true;
//                int write = SerialPortInterface.write(writeData, 0, writeData.length);
                boolean isPolled = false;
                byte[] resetRequest = createBytes();
                if(writeData(resetRequest)) {
                    boolean isReadReset = false;
                    boolean isReadSetupPrice = false;
                    boolean isReadEnableReader = false;
                    boolean isReadBeginSession = false;
                    boolean isReadVendDispatch = false;
                    boolean isReadConfig = false;
                    boolean isReadExpansionId = false;
                    boolean isReadVendApproved = false;
                    boolean isReadEnd = false;
                    boolean isWriteReset = false;
                    boolean isWriteSetupPrice = false;
                    boolean isWriteEnableReader = false;
                    boolean isWriteBeginSession = false;
                    boolean isWriteVendDispatch = false;
                    boolean isWriteConfig = false;
                    boolean isWriteExpansionId = false;
                    boolean isWriteVendApproved = false;
                    boolean isWriteEnd = false;
                    while (flagEnd) {
                        isPolled = false;
                        byte[] readResult = new byte[1];
                        int read = SerialPortInterface.read(readResult, readResult.length, 1000);
                        String sp = "";
                        if (read >= 0) {
                            if (ByteConvertStringUtil.buf2StringCompact(readResult).equals("09")) {
                                read = SerialPortInterface.read(readResult, readResult.length, 500);
                                if (read > 0) {
                                    String lenthHex = ByteConvertStringUtil.buf2StringCompact(readResult);
                                    BigInteger item = new BigInteger(lenthHex, 16);
                                    readResult = new byte[item.intValue() + 1];
                                    read = SerialPortInterface.read(readResult, readResult.length, 500);
                                    String stringCompact = ByteConvertStringUtil.buf2StringCompact(readResult);
                                    Log.d(TAG, "read success !!! read result : " + stringCompact);
                                    sp = stringCompact;
                                }
                            } else {
                                sp = "";
                                flagEnd = false;
                            }
                            if (sp.startsWith("01 00")) {
                                if(sp.equals("01 00 00 FF 0D")){
                                    isReadReset = true;
                                }
                                if (!isReadConfig && isReadReset) {
                                    //0011000200000114
                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x11, 0x00, 0x02, 0x00, 0x00, 0x01, 0x14});
                                    if(isWriteConfig){
                                        writeData(createPollBytes());
                                    }else if (writeData(wbytes)) {
                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending setup config request").sendToTarget();
                                        isWriteConfig = true;
                                    }
                                }else if (!isReadExpansionId && isReadSetupPrice) {
                                    //0017004944533030303031303030363630324944534d4442564d43434f4e303221
                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x17, 0x00, 0x49, 0x44, 0x53, 0x30, 0x30, 0x30, 0x30, 0x31, 0x30, 0x30, 0x30, 0x36, 0x36, 0x30,
                                            0x32, 0x49, 0x44, 0x53, 0x4D, 0x44, 0x42, 0x56, 0x4D, 0x43, 0x43, 0x4F, 0x4E, 0x30, 0x32, 0x21});
                                    if(isWriteExpansionId){
                                        writeData(createPollBytes());
                                    }else if (writeData(wbytes)) {
                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending Expansion ID request").sendToTarget();
                                        isWriteExpansionId = true;
                                    }
                                }else if (!isReadBeginSession && isReadEnableReader) {
                                    //00030064ffffffff00000063
//                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x03, 0x00, 0x64, (byte) 0xFF, (byte) 0xFF, (byte) 0xFF,
//                                            (byte) 0xFF, 0x00, 0x00, 0x00, 0x63});
//                                    if(isWriteBeginSession){
//                                        writeData(createPollBytes());
//                                    }else if (writeData(wbytes)) {
//                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending begin session with balance 0064 request").sendToTarget();
                                    isReadBeginSession = true;
                                    isWriteBeginSession = true;
//                                    }
                                }else if (!isReadVendApproved && isReadBeginSession) {
                                    //001300006400e158
                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x13, 0x00, 0x00, 0x64, 0x00, (byte) 0xE1, 0x58});
                                    if(isWriteVendApproved){
                                        writeData(createPollBytes());
                                    }else if (writeData(wbytes)) {
                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending vend request with amount 0064").sendToTarget();
                                        isWriteVendApproved = true;
                                    }
                                }else if (!isReadEnd && isReadVendDispatch) {
                                    //00130417
                                    byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x13, 0x04, 0x17});
                                    if(isWriteEnd){
                                        writeData(createPollBytes());
                                    }else if (writeData(wbytes)) {
                                        handler.obtainMessage(MDBUtils.BLACK_LOG, "sending vend session end request").sendToTarget();
                                        isReadEnd = true;
                                        isWriteEnd = true;
                                    }
                                }
                            }
                            if (sp.startsWith("01 01") && !isReadSetupPrice) {
                                isReadConfig = true;
                                //0011012b5c000aa3
                                byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x11, 0x01, 0x2B, 0x5C, 0x00, 0x0A, (byte) 0xA3});
                                if(isWriteSetupPrice){
                                    writeData(createPollBytes());
                                }else if (writeData(wbytes)) {
                                    handler.obtainMessage(MDBUtils.BLACK_LOG, "sending setup max/mix price request").sendToTarget();
                                    isReadSetupPrice = true;
                                    isWriteSetupPrice = true;
                                }
                            }
                            if (sp.startsWith("01 09") && !isReadEnableReader) {
                                isReadExpansionId = true;
                                //00140115
                                byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x14, 0x01, 0x15});
                                if(isWriteEnableReader){
                                    writeData(createPollBytes());
                                }else if (writeData(wbytes)) {
                                    handler.obtainMessage(MDBUtils.BLACK_LOG, "sending enable Reader request").sendToTarget();
                                    isReadEnableReader = true;
                                    isWriteEnableReader = true;
                                }
                            }
                            if (sp.startsWith("01 05") && !isReadVendDispatch) {
                                isReadVendApproved = true;
                                //00130200e1f6
                                byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x13, 0x02, 0x00, (byte) 0xE1, (byte) 0xF6});
                                if(isWriteVendDispatch){
                                    writeData(createPollBytes());
                                }else if (writeData(wbytes)) {
                                    handler.obtainMessage(MDBUtils.BLACK_LOG, "sending vend dispatch request with item 00e1").sendToTarget();
                                    isReadVendDispatch = true;
                                    isWriteVendDispatch = true;
                                }
                            }
                            if (sp.startsWith("01 06")) {
                                byte[] wbytes = MDBUtils.mergePacket(0, new byte[]{0x13, 0x04, 0x17});
                                if(isWriteEnd){
                                    writeData(createPollBytes());
                                }else if (writeData(wbytes)) {
                                    handler.obtainMessage(MDBUtils.BLACK_LOG, "vend denied!! sending vend session end request").sendToTarget();
                                    isReadEnd = true;
                                    isWriteEnd = true;
                                }
                            }
                            if (sp.startsWith("01 07")) {
                                flagEnd = false;
                            }
                        }else if(!isPolled){
                            if(writeData(createPollBytes())){
                                isPolled = true;
                            }
                        } else if (read == -262254) {
                            handler.obtainMessage(MDBUtils.RED_LOG, "serial port read timeout").sendToTarget();
                        } else {
                            handler.obtainMessage(MDBUtils.RED_LOG, "serial port read failed = " + read).sendToTarget();
                            Log.d(TAG, "read fail!!");
                        }
                    }
                }
                else {
                    Log.e(TAG, "write failed!! write data = " + ByteConvertStringUtil.buf2StringCompact(resetRequest));
                }
            }
        }).start();
    }

    /**
     * 1. test mib
     * send command to serial "Hardware diagnostic test":
     * Eg：
     * Send Data:090300946c0d
     * Recv Data: 09040194006b0d
     * Ps： ifCMDID（0x94）后面的ACK 不为0x00 ，说明MDB 通讯timeout
     * 2.get MIB version
     * send get version cmd
     * Send Data: 090300926e0d
     * Recv Data: 090601920000 0d 600d
     */

    void testMdb() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                open();
                byte[] bytes = {0x09, 0x03, 0x00, (byte) 0x94, (byte) 0x6C, (byte) 0x0D};
                int write = SerialPortInterface.write(bytes, 0, bytes.length);
                if (write >= 0) {
                    handler.obtainMessage(MDBUtils.BLACK_LOG, "testing").sendToTarget();
                    byte[] resulta = new byte[16];
                    int read = SerialPortInterface.read(resulta, resulta.length, 3000);
                    if (read >= 0) {
                        String bytes2Str = ByteConvertStringUtil.buf2StringCompact(subByteArray(resulta, read));
                        if (bytes2Str.startsWith("[09 04 01 94 00")) {
                            handler.obtainMessage(MDBUtils.BLACK_LOG, "read ok: " + bytes2Str).sendToTarget();
                            handler.obtainMessage(MDBUtils.GREEN_LOG, "test ok").sendToTarget();
                        } else {
                            handler.obtainMessage(MDBUtils.RED_LOG, "MDB read timeout! " + bytes2Str).sendToTarget();
                            handler.obtainMessage(MDBUtils.RED_LOG, "test failed!").sendToTarget();
                        }
                    } else if (read == -262254) {
                        handler.obtainMessage(MDBUtils.RED_LOG, "serialport read timeout").sendToTarget();
                    } else {
                        handler.obtainMessage(MDBUtils.RED_LOG, "serialport read failed").sendToTarget();
                    }
                }
                close();
            }
        }).start();
    }

    //e.g. version 0019: sp == 01 92 00 00 13 5A, 00 13 decides 19
    public void printVersionName(byte[] readBytes){
        int high = readBytes[3];
        int low = readBytes[4];
        currentVersion = ((high & 0xFF) << 8) | (low & 0xFF);
        handler.obtainMessage(MDBUtils.GREEN_LOG, "version = " + currentVersion).sendToTarget();
    }

    //可以添加参数 byte[] contentBytes
    private byte[] createBytes(){
        int lrc;
        byte[] contentBytes = {0x00, 0x10, 0x10};
        lrc = LRCCheckInt(contentBytes);
        byte[] dataBytes = {0x09, 0x04, 0x00, 0x10, 0x10, (byte) lrc, (byte) 0x0D};
        return dataBytes;
    }

    private byte[] createPollBytes(){
        int lrc;
        byte[] contentBytes = {0x00, 0x12, 0x12};
        lrc = LRCCheckInt(contentBytes);
        byte[] dataBytes = {0x09, 0x04, 0x00, 0x12, 0x12, (byte) lrc, (byte) 0x0D};
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
        return dataBytes;
    }

    private boolean writeData(byte[] data){
        boolean ret = false;
        int write = SerialPortInterface.write(data, 0, data.length);
        if(Arrays.equals(data, createPollBytes())){
            Log.d(TAG, "polling ...");
        } else if(write >= 0){
            Log.d(TAG, "write data success! write length : " + data.length + "data : " + ByteConvertStringUtil.buf2StringCompact(data));
            ret = true;
        }else{
            Log.e(TAG, "write data failed!! Data : " + ByteConvertStringUtil.buf2StringCompact(data));
        }
        return ret;
    }
    boolean isDownload;

    Thread downloadTh;

    byte[] readSDfile(String sdcard_mib_path) {
        File file = new File(sdcard_mib_path);
        try {
            FileInputStream fis = new FileInputStream(file);
            int ch = 0;
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            while ((ch = fis.read()) != -1) {
                baos.write(ch);
            }
            baos.flush();
            baos.close();
            fis.close();
            return baos.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private void updateFw() {
        if (downloadTh != null && downloadTh.getState() != Thread.State.TERMINATED) {
            Log.d(TAG, "downloadFw  return" + " downloadFw  return");
            handler.obtainMessage(MDBUtils.BLACK_LOG, "downloadFw  return").sendToTarget();
            return;
        }
        listAssets();
        downloadTh = new Thread(new Runnable() {
            @Override
            public void run() {
                isDownload = true;
                SerialPortInterface.close();
                int open = Stm32ispInterface.ispOpen();
                if (open >= 0) {
                    handler.obtainMessage(MDBUtils.BLACK_LOG, "stm32 open success").sendToTarget();
                    try {
                        String binPath = Environment.getExternalStorageDirectory().getPath() + "/mib_test.bin";
                        byte[] fwTotal = readSDfile(binPath);
                        if (fwTotal != null && fwTotal.length > 100) {
                            handler.obtainMessage(MDBUtils.BLACK_LOG, "\nfound sdcard mib firmware ...,\n, " + binPath).sendToTarget();
                        } else {
//                            binPath = assetFiles.get(assetFiles.size() - 1);
                            binPath = getBinName();
                            if(binPath != null){
                                handler.obtainMessage(MDBUtils.BLACK_LOG, "\nfound mib firmware ...,\n, " + binPath).sendToTarget();
                                fwTotal = toByteArray(binPath);
                            }else{
                                handler.obtainMessage(MDBUtils.BLACK_LOG, "\nmib firmware not found!\n ").sendToTarget();
                            }
                        }
//                        if (!fwName.contains("facttest")) {
                        fwTotal = Arrays.copyOfRange(fwTotal, 0x80, fwTotal.length);
//                        }

                        float length = fwTotal.length;
                        handler.obtainMessage(MDBUtils.BLACK_LOG, "\ndownloading mib firmware...,\n name= " + binPath + ", size= " + length).sendToTarget();
                        byte[][] bytes1 = MDBUtils.splitBytes(fwTotal, 0x100);
                        int count = 0;
                        for (byte[] b : bytes1) {
                            Stm32ispInterface.ispDownload(b, b.length);
                            float writeLength = count += b.length;
                            float f = writeLength / length;
                            handler.obtainMessage(3, f * 100).sendToTarget();
                        }
                        handler.obtainMessage(MDBUtils.GREEN_LOG, "\ndownload complete!").sendToTarget();
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        int i = Stm32ispInterface.ispClose();
                        if (i >= 0) {
                            handler.obtainMessage(1, "stm32 close success").sendToTarget();
                        } else {
                            handler.obtainMessage(2, "stm32 close failed").sendToTarget();
                        }
                        isDownload = false;
                    }
                    SystemClock.sleep(100);
                    int result = SerialPortInterface.open("SERIAL_EXT");
                    sendVersionRequest();
                } else {
                    handler.obtainMessage(2, "stm32 open failed").sendToTarget();
                    isDownload = false;
                    Stm32ispInterface.ispClose();
                }
            }
        });
        downloadTh.start();
    }

    ArrayList<String> assetFiles = new ArrayList<>();

    private void listAssets() {
        String[] list = new String[0];
        try {
            list = context.getAssets().list("");
        } catch (IOException e) {
            e.printStackTrace();
        }
        for (String fileName : list) {
            if (fileName.endsWith(".bin")) {//fileName.startsWith("mib") &&
//                Log.d(TAG, "assetFile=" + fileName);
                assetFiles.add(fileName);
            }
        }
    }

    private String getBinName(){
        listAssets();
        String file_name;
        switch (firmwareName){
            case "0014":
                file_name = "WhiteDemon_0014.bin";
                break;
            case "0015":
                file_name = "WhiteDemon_0015.bin";
                break;
            case "0017":
                file_name = "WhiteDemon_0017.bin";
                break;
            case "0018":
                file_name = "WhiteDemon_0018.bin";
                break;
            case "0018_debug":
                file_name = "WhiteDemon_debug_0018.bin";
                break;
            case "0019":
                file_name = "WhiteDemon_0019.bin";
                break;
            case "0020":
                file_name = "WhiteDemon_0020.bin";
                break;
            case "0021":
                file_name = "WhiteDemon_0021.bin";
                break;
            case "0022":
                file_name = "WhiteDemon_0022.bin";
                break;
            default:
                file_name = null;
        }
        return file_name;
    }

    private int setPulse(){
        int ret = -1;
        if(pulseDuration < 0){
            return ret;
        }
        if(pulseVoltage != 0 && pulseVoltage != 1){
            return ret;
        }
        if(pulseLatency <= 0){
            return ret;
        }
        if(pulseFrequency <= 0){
            return ret;
        }
        sendSetPulseDuration(pulseDuration);
        sendSetPulseVoltage(pulseVoltage);
        return 1;
    }

    public byte[] toByteArray(String fname) throws IOException {
        InputStream fis = context.getAssets().open(fname);
        int ch = 0;
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        while ((ch = fis.read()) != -1) {
            baos.write(ch);
        }
        baos.flush();
        baos.close();
        fis.close();
        return baos.toByteArray();
    }


    @Override
    public void onDestroy() {
        super.onDestroy();
        close();
    }

    void open() {
        int result = SerialPortInterface.open("SERIAL_EXT");
        if (result < 0) {
            onLogSent('e', TAG,"seriaport open failed!" + result);
        } else {
            result = SerialPortInterface.setBaudrate(115200);
            SystemClock.sleep(100);
            handler.obtainMessage(MDBUtils.BLACK_LOG, "seriaport open success!").sendToTarget();
//            if(isSlave)
            testMdbSlaveRead();
        }
    }
    void close() {
        int result = SerialPortInterface.close();
        if (result < 0) {
            Log.e(TAG, "SerialPort close failed : " + result);
            handler.obtainMessage(MDBUtils.RED_LOG, "seriaport close failed " + result).sendToTarget();
        } else {
            handler.obtainMessage(MDBUtils.BLACK_LOG, "seriaport close success").sendToTarget();
        }
    }

    public void test(){
        // read: 00 13 00 00 A0 00 01 B4 98, after read and touch card
        // approved! write: 09 06 01 05 00 A0 A5 B5 0D
        byte[] wbytes = {0x09, 0x06, 0x01, 0x05, 0x00, (byte)0xA0, (byte)0xA5, (byte)0xB5, 0x0D};
        // read: 00 13 02 00 11 26 B4
        // Dispatch success! write: 09 03 01 00 FF 0D
//        byte[] wbytes = {0x09, 0x03, 0x01, 0x00, (byte)0xFF, 0x0D};

        int wresult = SerialPortInterface.write(wbytes, 0, wbytes.length);
        if (wresult >= 0) {
            Log.d(TAG, "test wbytes:" + ByteConvertStringUtil.buf2StringCompact(wbytes));
        }
    }

    public static byte[] subByteArray(byte[] byteArray, int length) {
        byte[] arrySub = new byte[length];
        if (length >= 0) System.arraycopy(byteArray, 0, arrySub, 0, length);
        return arrySub;
    }

    public static byte[] subByteArrayIgnore(byte[] byteArray, int startIndex, int length) {
        int realLength = length - startIndex;
        byte[] arrySub = new byte[realLength];
        if (length >= 0) System.arraycopy(byteArray, startIndex, arrySub, 0, realLength);
        return arrySub;
    }

    private byte[] createBytes(String edit) {
        ArrayList<Byte> as = new ArrayList<>();

        int s1 = 9;
        int e1 = 13;

        int len;
        int lrc;
        String[] split = edit.replace(",", " ").split(" ");
        len = split.length + 1;
        for (String spli : split) {
            as.add((byte) Integer.parseInt(spli, 16));
        }
        byte[] by = new byte[as.size()];
        for (int i = 0; i < as.size(); i++) {
            by[i] = as.get(i);
        }
        lrc = LRCCheckInt(by);
        ArrayList<Byte> as2 = new ArrayList<>();

        as2.add((byte) s1);
        as2.add((byte) len);

        for (byte b : by) {
            as2.add(b);
        }

        as2.add((byte) lrc);
        as2.add((byte) e1);

        byte[] by2 = new byte[as2.size()];
        for (int i = 0; i < as2.size(); i++) {
//            Log.d("ByteConvertStringUtil.buf2StringCompact by2","as2.get(i)" +as2.get(i));
            by2[i] = as2.get(i);
        }

        String s = ByteConvertStringUtil.buf2StringCompact(by2);
        Log.d(TAG, "ss = " + s);
        return by2;
    }

    private int LRCCheckInt(byte[] bytes) {
        int l1 = 0;
        for (byte b : bytes) {
            l1 = l1 + (int) b;
        }
        return ~l1 + 1;
    }

    private String LRCCheck(byte[] bytes) {
        int l1 = 0;
        for (byte b : bytes) {
            l1 = l1 + (int) b;
        }
        return Integer.toHexString(~l1 + 1);
    }

    public static String getTimeWithFormat() {
        long time = System.currentTimeMillis();
        SimpleDateFormat sdr = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.CHINA);
        return sdr.format(time);
    }

    @Override
    public void onBackPressed() {
        if(drawerLayout.isDrawerOpen(GravityCompat.START)){
            drawerLayout.closeDrawer(GravityCompat.START);
        }else {
            super.onBackPressed();
        }
    }

    @Override
    public void onLogSent(char c, String tag, String log) {
        switch (c){
            case 'd':
                handler.obtainMessage(MDBUtils.BLACK_LOG, log).sendToTarget();
                Log.d(tag, log);
                break;
            case 'i':
                handler.obtainMessage(MDBUtils.BLACK_LOG, log).sendToTarget();
                Log.i(tag, log);
                break;
            case 'e':
                handler.obtainMessage(MDBUtils.RED_LOG, log).sendToTarget();
                Log.e(tag, log);
                break;
        }
    }

    @Override
    public void onIntValueSent(int type, int value) {
        switch(type){
            case MDBUtils.TYPE_PULSE_LATENCY:
                pulseLatency = value;
                break;
            case MDBUtils.TYPE_PULSE_FREQUENCY:
                pulseFrequency = value;
                break;
            case MDBUtils.TYPE_PULSE_DURATION:
                pulseDuration = value;
                break;
            case MDBUtils.TYPE_PULSE_VOLTAGE:
                pulseVoltage = value;
                break;
            case MDBUtils.TYPE_BALANCE:
                balance = value;
                break;
            case MDBUtils.TYPE_SPN_MDB_LEVEL:
                mdbLevel = value;
                break;

            default:
                break;
        }
    }

    @Override
    public void onStringValueSent(int type, String value) {
        switch (type){
            case MDBUtils.TYPE_SPN_FIRMWARE_ITEM:
                firmwareName = value;
                break;
            default:
                break;
        }
    }

    @Override
    public void onOpenClicked() {
        open();
    }

    @Override
    public void onCloseClicked() {
        close();
    }

    @Override
    public void onGetVersionClicked() {
        sendVersionRequest();
    }

    @Override
    public void onUpdateFwClicked() {
        updateFw();
    }

    @Override
    public void onTestClicked() {
        test();
    }

    Handler delayHandler = new Handler();

    @Override
    public void onTriggerPulseClicked() {
        if(pulseFrequency < 0){
            return;
        }
        currentSend = pulseFrequency;
        if(pulseLatency < 0){
            return;
        }
        handler.obtainMessage(MDBUtils.DISABLE_ALL_UI).sendToTarget();
        sendVersionRequest();
        if(setPulse() < 0){
            Log.e(TAG, "pulse parameter error!");
            handler.obtainMessage(MDBUtils.BLACK_LOG, "pulse parameter error!").sendToTarget();
            return;
        }
        SystemClock.sleep(500);
        if(currentVersion >= 19){
            if(!TRIGGER_PULSE_ONE_SHOT) {
                handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse processing, please wait ...").sendToTarget();
                sendTriggerPulse();
                for (int i = 1; i < pulseFrequency; ++i) {
                    int delayTime = i * (pulseLatency + pulseDuration);
                    delayHandler.postDelayed(new Runnable() {
                        @Override
                        public void run() {
                            sendTriggerPulse();
                        }
                    }, delayTime);
                }
            } else {
                if(currentVersion >= 22) {
                    handler.obtainMessage(MDBUtils.BLACK_LOG, "trigger pulse processing, please wait ...").sendToTarget();
                    sendTriggerPulseOneShot();
                    delayHandler.postDelayed(new Runnable() {
                        @Override
                        public void run() {
                            handler.obtainMessage(MDBUtils.ENABLE_ALL_UI).sendToTarget();
                        }
                    }, pulseFrequency * (pulseLatency + pulseDuration));
                }
                else {
                    handler.obtainMessage(MDBUtils.RED_LOG, "version is not equal or greater than 22, " +
                            "trigger pulse one shot failed, please update FW version").sendToTarget();
                }
            }
        }else{
            handler.obtainMessage(MDBUtils.RED_LOG, "version is not equal or greater than 19, trigger pulse failed, " +
                    "please update FW version").sendToTarget();
        }
    }

    @Override
    public void onSetPulseClicked() {
        if(setPulse() < 0){
            Log.e(TAG, "pulse parameter error!");
            handler.obtainMessage(MDBUtils.BLACK_LOG, "pulse parameter error!").sendToTarget();
        }
    }

    @Override
    public void onFactoryModeClicked() {
        View view = selectedFragment.getView();
        if(view != null){
            Button button = view.findViewById(R.id.btn_factory_mode);
            String buttonText = button.getText().toString();
            if(buttonText.equalsIgnoreCase("enter factory mode")){
                sendEnterFactoryMode();
            } else if (buttonText.equalsIgnoreCase("quit factory mode")) {
                sendQuitFactoryMode();
            }
        }
    }

    @Override
    public void onGetHardwareVersionClicked() {
        sendHardwareVersionRequest();
    }

    @Override
    public void onDiagnoseHardwareClicked() {
        sendHardwareDiagnoseTest();
    }

    public void printLogAndText(char c, String s){
        onLogSent(c, TAG, s);
    }

    private void disableAllUi(){
        NavigationView navigationView = findViewById(R.id.nav_view);
        MenuItem menuItem = navigationView.getCheckedItem();
        if(menuItem != null){
            int itemId = menuItem.getItemId();
            if(itemId == R.id.nav_home){
                disableHomeUi();
            } else if(itemId == R.id.nav_pulse){
                disablePulseUi();
            } else if(itemId == R.id.nav_transaction){
                disableTransactionUi();
            }
        }
    }

    private void enableAllUi(){
        NavigationView navigationView = findViewById(R.id.nav_view);
        MenuItem menuItem = navigationView.getCheckedItem();
        if(menuItem != null){
            int itemId = menuItem.getItemId();
            if(itemId == R.id.nav_home){
                enableHomeUi();
            } else if(itemId == R.id.nav_pulse){
                enablePulseUi();
            } else if(itemId == R.id.nav_transaction){
                enableTransactionUi();
            }
        }
    }

    private void disablePulseUi(){
        EditText etDuration = selectedFragment.getView().findViewById(R.id.et_duration);
        etDuration.setEnabled(false);
        EditText etFrequency = selectedFragment.getView().findViewById(R.id.et_frequency);
        etFrequency.setEnabled(false);
        EditText etLatency = selectedFragment.getView().findViewById(R.id.et_latency);
        etLatency.setEnabled(false);

        Button btnTriggerPulse = selectedFragment.getView().findViewById(R.id.btn_trigger_pulse);
        btnTriggerPulse.setEnabled(false);
        Button btnSetPulse = selectedFragment.getView().findViewById(R.id.btn_set_pulse);
        btnSetPulse.setEnabled(false);

        Spinner spnVoltage = selectedFragment.getView().findViewById(R.id.spn_voltage);
        spnVoltage.setEnabled(false);
    }

    private void enablePulseUi(){
        EditText etDuration = selectedFragment.getView().findViewById(R.id.et_duration);
        etDuration.setEnabled(true);
        EditText etFrequency = selectedFragment.getView().findViewById(R.id.et_frequency);
        etFrequency.setEnabled(true);
        EditText etLatency = selectedFragment.getView().findViewById(R.id.et_latency);
        etLatency.setEnabled(true);

        Button btnTriggerPulse = selectedFragment.getView().findViewById(R.id.btn_trigger_pulse);
        btnTriggerPulse.setEnabled(true);
        Button btnSetPulse = selectedFragment.getView().findViewById(R.id.btn_set_pulse);
        btnSetPulse.setEnabled(true);

        Spinner spnVoltage = selectedFragment.getView().findViewById(R.id.spn_voltage);
        spnVoltage.setEnabled(true);
    }

    private void disableTransactionUi(){
        Button btnSetBalance = selectedFragment.getView().findViewById(R.id.btn_set_balance);
        btnSetBalance.setEnabled(false);
        EditText etBalance = selectedFragment.getView().findViewById(R.id.et_balance);
        etBalance.setEnabled(false);
    }

    private void enableTransactionUi(){
        Button btnSetBalance = selectedFragment.getView().findViewById(R.id.btn_set_balance);
        btnSetBalance.setEnabled(true);
        EditText etBalance = selectedFragment.getView().findViewById(R.id.et_balance);
        etBalance.setEnabled(true);
    }

    private void disableHomeUi(){
        Button btnGetVersion = selectedFragment.getView().findViewById(R.id.btn_get_version);
        btnGetVersion.setEnabled(false);
        Button btnUpdateFw = selectedFragment.getView().findViewById(R.id.btn_update_fw);
        btnUpdateFw.setEnabled(false);
        Button btnFactoryMode = selectedFragment.getView().findViewById(R.id.btn_factory_mode);
        btnFactoryMode.setEnabled(false);
        Button btnGetHardwareVersion = selectedFragment.getView().findViewById(R.id.btn_get_hardware_version);
        btnGetHardwareVersion.setEnabled(false);
        Spinner spnWhiteDemon = selectedFragment.getView().findViewById(R.id.spn_white_demon);
        spnWhiteDemon.setEnabled(false);
        Spinner spnMdbLevel = selectedFragment.getView().findViewById(R.id.spn_mdb_level);
        spnMdbLevel.setEnabled(false);
    }

    private void enableHomeUi(){
        Button btnGetVersion = selectedFragment.getView().findViewById(R.id.btn_get_version);
        btnGetVersion.setEnabled(true);
        Button btnUpdateFw = selectedFragment.getView().findViewById(R.id.btn_update_fw);
        btnUpdateFw.setEnabled(true);
        Button btnFactoryMode = selectedFragment.getView().findViewById(R.id.btn_factory_mode);
        btnFactoryMode.setEnabled(true);
        Button btnGetHardwareVersion = selectedFragment.getView().findViewById(R.id.btn_get_hardware_version);
        btnGetHardwareVersion.setEnabled(true);
        Spinner spnWhiteDemon = selectedFragment.getView().findViewById(R.id.spn_white_demon);
        spnWhiteDemon.setEnabled(true);
        Spinner spnMdbLevel = selectedFragment.getView().findViewById(R.id.spn_mdb_level);
        spnMdbLevel.setEnabled(true);
    }

}