package com.wizarpos.util;

import java.util.Arrays;

public class MDBUtils {

	public static final int BLACK_LOG = 1;
	public static final int RED_LOG = 2;
	public static final int GREEN_LOG = 4;
	public static final int DIALOG_CONFIRM = 8;
	public static final int DIALOG_SET_PULSE = 9;
	public static final int DIALOG_ITEM = 10;
	public static final int DIALOG_READCARD = 11;
	public static final int ENABLE_ALL_UI = 15;
	public static final int DISABLE_ALL_UI = 16;

	public static final int TYPE_PULSE_LATENCY = 20;
	public static final int TYPE_PULSE_FREQUENCY = 21;
	public static final int TYPE_PULSE_DURATION = 22;
	public static final int TYPE_PULSE_VOLTAGE = 23;
	public static final int TYPE_BALANCE = 24;
	public static final int TYPE_SPN_FIRMWARE_ITEM = 25;
	public static final int TYPE_SPN_MDB_LEVEL = 26;

	public static final int TEST_MSG = 99;

	public static byte[] addMdbCheckSum(byte[] buf) {
		for(int i = 0; i < buf.length-1; i++){
			buf[buf.length-1] += buf[i];
		}
		return buf;
	}

	/*
	 * add 09 before data,add checksum and 0D after length and mode
	 * data: data content
	 * mode: 00: request, 01: response
	 */
	public static byte[] mergePacket(int mode, byte[] data) {
		byte startCode = 0x09;
		byte endCode = 0x0d;
		byte modebyte = 0x00;
		if (mode == 1) {
			modebyte = 0x01;
		}
		//------
		byte[] modeANDdata = new byte[1 + data.length];
		modeANDdata[0] = modebyte;
		System.arraycopy(data, 0, modeANDdata, 1, data.length);
		byte checkSumLRC = getLRC(modeANDdata);
		//------
		byte[] mergeBytes = new byte[1 + 1 + 1 + data.length + 1 + 1];
		mergeBytes[0] = startCode;
		mergeBytes[1] = (byte) (1 + 1 + data.length);
		mergeBytes[2] = modebyte;
		System.arraycopy(data, 0, mergeBytes, 3, data.length);
		mergeBytes[mergeBytes.length - 2] = checkSumLRC;
		mergeBytes[mergeBytes.length - 1] = endCode;
		//String.format("%02X", byte)
		return mergeBytes;
	}

	public static byte getLRC(byte[] buf) {
		int rr = 0x00;
		for (byte b : buf) {
			rr += (b & 0xFF);
		}
		rr = ~rr + 1;
		return (byte) rr;
	}

	/**
	 * split byte[]
	 */
	public static byte[][] splitBytes(byte[] bytes, int size) {
		double splitLength = Double.parseDouble(size + "");
		int arrayLength = (int) Math.ceil(bytes.length / splitLength);
		byte[][] result = new byte[arrayLength][];
		int from, to;
		for (int i = 0; i < arrayLength; i++) {

			from = (int) (i * splitLength);
			to = (int) (from + splitLength);
			if (to > bytes.length)
				to = bytes.length;
			result[i] = Arrays.copyOfRange(bytes, from, to);
		}
		return result;
	}

}
